import { useState } from 'react';
import { useDebounce } from './useDebounce.js';
import { useDidMount } from './useDidMount.js';
import { useDidUpdate } from './useDidUpdate.js';

const defaultUseDebounceValueOptions = {
    initializeWithNull: false,
};
/**
 * useDebouncedValue
 * @param value The value to debounce
 * @param timeout The duration to debounce
 * @param options The options object.
 * @see https://rooks.vercel.app/docs/useDebouncedValue
 */
const useDebouncedValue = (value, timeout, options = {}) => {
    // eslint-disable-next-line prefer-object-spread
    const { initializeWithNull } = Object.assign({}, defaultUseDebounceValueOptions, options);
    const [updatedValue, setUpdatedValue] = useState(initializeWithNull ? null : value);
    const debouncedSetUpdatedValue = useDebounce(setUpdatedValue, timeout);
    useDidMount(() => {
        if (initializeWithNull) {
            debouncedSetUpdatedValue(value);
        }
    });
    useDidUpdate(() => {
        debouncedSetUpdatedValue(value);
    }, [value]);
    // No need to add `debouncedSetUpdatedValue ` to dependencies as it is a ref.current.
    // returning both updatedValue and setUpdatedValue (not the debounced version) to instantly update this if  needed.
    return [updatedValue, setUpdatedValue];
};

export { useDebouncedValue };
