import { useState, useCallback } from 'react';

/**
 * useNativeMapState
 * @description Manage Map() object state in React
 * @see {@link https://rooks.vercel.app/docs/useNativeMapState}
 */
function useNativeMapState(initialMapState) {
    const [map, setMap] = useState(initialMapState || new Map());
    const set = useCallback((key, value) => {
        setMap((prevMap) => new Map(prevMap.set(key, value)));
    }, []);
    const setMultiple = useCallback((entries) => {
        setMap((prevMap) => {
            const newMap = new Map(prevMap);
            entries.forEach(([key, value]) => {
                newMap.set(key, value);
            });
            return newMap;
        });
    }, []);
    const remove = useCallback((key) => {
        setMap((prevMap) => {
            const newMap = new Map(prevMap);
            newMap.delete(key);
            return newMap;
        });
    }, []);
    const removeMultiple = useCallback((keys) => {
        setMap((prevMap) => {
            const newMap = new Map(prevMap);
            keys.forEach((key) => {
                newMap.delete(key);
            });
            return newMap;
        });
    }, []);
    const clear = useCallback(() => {
        setMap(new Map());
    }, []);
    const has = useCallback((key) => {
        return map.has(key);
    }, [map]);
    const hasSome = useCallback((keys) => {
        return keys.some((key) => map.has(key));
    }, [map]);
    const hasEvery = useCallback((keys) => {
        return keys.every((key) => map.has(key));
    }, [map]);
    const get = useCallback((key) => {
        return map.get(key);
    }, [map]);
    const size = useCallback(() => {
        return map.size;
    }, [map]);
    const controls = {
        set,
        remove,
        clear,
        has,
        get,
        size,
        setMultiple,
        removeMultiple,
        hasSome,
        hasEvery,
    };
    return [map, controls];
}

export { useNativeMapState };
