import { noop } from '../utils/noop.js';
import { useState, useCallback, useEffect } from 'react';
import { useFreshCallback } from './useFreshCallback.js';

/**
 * useOnClickRef hook
 *
 * This hook runs a callback for both clicks and tap events when the element is clicked or tapped.
 *
 * @param {Function} onClick The callback function to run on click or tap
 * @returns {Function} A callback ref which can be attached to an element
 */
function useOnClickRef(onClick) {
    const [node, setNode] = useState(null);
    const handleEvent = useCallback((event) => {
        event.preventDefault();
        onClick();
    }, [onClick]);
    const freshCallback = useFreshCallback(handleEvent);
    useEffect(() => {
        if (node) {
            node.addEventListener("click", freshCallback);
            node.addEventListener("touchend", freshCallback);
            return () => {
                node.removeEventListener("click", freshCallback);
                node.removeEventListener("touchend", freshCallback);
            };
        }
        return noop;
    }, [node, freshCallback]);
    const ref = useCallback((element) => {
        if (element) {
            setNode(element);
        }
    }, []);
    return ref;
}

export { useOnClickRef };
